#ifndef _UOSDEVICEH_V2_RAW_H_
#define _UOSDEVICEH_V2_RAW_H_

#ifdef __cplusplus
extern "C" {
#endif

#define UOSDEVICEH_VERSION 1 //1-255

#define SIG_HWINFO_LEN 128 // for temporary definition


/**********************************
 * @brief MSDM的oem信息获取需要的数据结构
 * 
 */
#define MSDMFILENAME "/sys/firmware/acpi/tables/MSDM" //LPIT  MSDM
#define BUFSIZE (56)    //(3 * 16 + 8)
#define SERIALSIZE (29)
typedef struct _MSDM
{
    char buf[BUFSIZE];      //其他预留信息
    char serial[SERIALSIZE]; //oem的序列号
} uoshwinfo_v2_raw_msdm;
//********************************


/**********************************
 * @brief 获取根目录所在硬盘
 * 
 */
#define DIRNAMELEN (2048)
#define ROOTDIRNAMEMAXLEN (2048)
#define ROOTNAMEPOSITION (4) 
#define DISKNAMEPOSITION (9)
#define MOUNTFILE "/proc/self/mountinfo"
#define CMDLINE "/proc/cmdline"
#define CMDLINE_ROOT "root="
#define CMDLINE_UUID "UUID="
#define CMDLINE_uuid "uuid="
typedef struct RootDirInfo uoshwinfo_v2_raw_rootdisk;
struct RootDirInfo
{
    char name[ROOTDIRNAMEMAXLEN];
};

typedef struct UuidInfo uoshwinfo_v2_raw_uuidinfo;
struct UuidInfo
{
    char devName[ROOTDIRNAMEMAXLEN];
    char uuid[ROOTDIRNAMEMAXLEN];
    char partUuid[ROOTDIRNAMEMAXLEN];
};
//********************************

typedef struct uoshwinfo_v2_raw_hwdisk {

  char product[SIG_HWINFO_LEN];
  char serial[SIG_HWINFO_LEN];
  char version[SIG_HWINFO_LEN];
  char name[SIG_HWINFO_LEN];
  unsigned long long size;
  unsigned char is_root;
  unsigned char removable;
 
} uoshwinfo_v2_raw_hwdisk;

// bios info
typedef struct uoshwinfo_v2_raw_bios {

  char vendor[SIG_HWINFO_LEN];
  char version[SIG_HWINFO_LEN];
  char date[SIG_HWINFO_LEN];
  char oeminfo[SIG_HWINFO_LEN];

} uoshwinfo_v2_raw_bios;

// machine system info
typedef struct uoshwinfo_v2_raw_system {

  char product[SIG_HWINFO_LEN];
  char vendor[SIG_HWINFO_LEN];
  char serial[SIG_HWINFO_LEN];
  char version[SIG_HWINFO_LEN];
  char uuid[SIG_HWINFO_LEN];

} uoshwinfo_v2_raw_system;

// machine board info
typedef struct uoshwinfo_v2_raw_board {

  char product[SIG_HWINFO_LEN];
  char vendor[SIG_HWINFO_LEN];
  char serial[SIG_HWINFO_LEN];
  char version[SIG_HWINFO_LEN];

} uoshwinfo_v2_raw_board;

// cpu info  //存在多个情况
typedef struct uoshwinfo_v2_raw_cpu {

  char product[SIG_HWINFO_LEN];
  char vendor[SIG_HWINFO_LEN];
  char serial[SIG_HWINFO_LEN];
  char id[SIG_HWINFO_LEN]; // compatible old version
  char version[SIG_HWINFO_LEN];

} uoshwinfo_v2_raw_cpu;

// memery info //存在多个情况
typedef struct uoshwinfo_v2_raw_mem {

  char product[SIG_HWINFO_LEN];
  char vendor[SIG_HWINFO_LEN];
  char serial[SIG_HWINFO_LEN];
  unsigned long long speed;
  unsigned long long size;

} uoshwinfo_v2_raw_mem;

// net devices info  //存在多个情况
typedef struct uoshwinfo_v2_raw_net {

  // //0:有线物理板载网卡，1：无线物理板载网卡，2：usb网卡，3：虚拟网卡
  char version[SIG_HWINFO_LEN];
  char mac[SIG_HWINFO_LEN]; //00:00:00:00:00:00 for error
  // char description[SIG_HWINFO_LEN]; //后续跟踪
  char name[SIG_HWINFO_LEN];
  unsigned char is_virtual; //0:not virt, 1:is virt, 0xff:error
  unsigned char index;    //0:error, other:index
  unsigned char enable;   //1:enable, 0:disable, 0xff:error


} uoshwinfo_v2_raw_net;

// raid info  //存在多个情况
typedef struct {
  unsigned char hwraid_status;
  unsigned char vroc_status;

} uoshwinfo_v2_raw_hwraid;

// virtual machine info //非源码获取内容
typedef struct uoshwinfo_v2_raw_vm {

  char type[SIG_HWINFO_LEN]; // vmware，qemu，virtual box ...

} uoshwinfo_v2_raw_vm;

typedef struct uoshwinfo_v2_raw_version {

  unsigned int version; //1-255

} uoshwinfo_v2_raw_version;

//====================if.h=================
#define UOS_HWINFO_HWDISK_MAX 64
#define UOS_HWINFO_CPU_MAX 128
#define UOS_HWINFO_MEM_DEV_MAX 64
#define UOS_HWINFO_NET_CARD_MAX 64

typedef struct __uoslshwinfo__
{
	uoshwinfo_v2_raw_system systeminfo;

    uoshwinfo_v2_raw_board boardinfo;

    uoshwinfo_v2_raw_bios biosinfo;

    uoshwinfo_v2_raw_cpu cpuinfo[UOS_HWINFO_CPU_MAX];

    uoshwinfo_v2_raw_mem meminfo[UOS_HWINFO_MEM_DEV_MAX + 1]; //total 1mem, for mips sw no bank

    uoshwinfo_v2_raw_net netinfo[UOS_HWINFO_NET_CARD_MAX];

	uoshwinfo_v2_raw_hwdisk hwdiskinfo[UOS_HWINFO_HWDISK_MAX];

	uoshwinfo_v2_raw_hwraid hwraidinfo;

	int hwdisk_num;
    int cpu_num;
    int net_num;
    int mem_num;

}uoshwinfo_lshw;


typedef struct _uoshwinfo_
{  
    uoshwinfo_lshw  lshwinfo;
   
}uoshwinfo_pool;

int uoshwinfo_dev_init();
void uoshwinfo_dev_exit();
//====================if.h= end================





int uoshwinfo_v2_raw_get_hwdisk_num();
void uoshwinfo_v2_raw_get_hwdisk(uoshwinfo_v2_raw_hwdisk *phwdiskinfo);
void uoshwinfo_v2_raw_get_hwraid(uoshwinfo_v2_raw_hwraid *phwraidinfo);
int uoshwinfo_util_get_nvmeinfo(uoshwinfo_v2_raw_hwdisk *pdiskinfo, int *pdisknum);
int uoshwinfo_detect_virt(uoshwinfo_v2_raw_vm *pvminfo);

/*Function Des:获取当前设备BIOS信息
  Input:N/A
  Output:pbiosinfo--主机信息数据结构；如果获取失败则数据内容为空
  Return:N/A
*/
void uoshwinfo_v2_raw_get_bios(uoshwinfo_v2_raw_bios *pbiosinfo);

/*Function Des:获取当前设备BOARD信息
  Input:N/A
  Output:pboardinfo--主机信息数据结构；如果获取失败则数据内容为空
  Return:N/A
*/
void uoshwinfo_v2_raw_get_board(uoshwinfo_v2_raw_board *pboardinfo);

/*Function Des:获取当前设备主机信息
  Input:N/A
  Output:psysteminfo--主机信息数据结构；如果获取失败则数据内容为空
  Return:N/A
*/
void uoshwinfo_v2_raw_get_system(uoshwinfo_v2_raw_system *psysteminfo);

/*Function Des:获取当前设备cpu数量
  Input:N/A
  Output:N/A
  Return:-1:系统错误， 其他则为cpu数量
*/
int uoshwinfo_v2_raw_get_cpu_num();

/*Function Des:获取当前设备CPU信息
  Input:N/A
  Output:pcpuinfo--CPU信息数据结构；如果获取失败则数据内容为空
  Return:N/A
*/
void uoshwinfo_v2_raw_get_cpu(uoshwinfo_v2_raw_cpu *pcpuinfo);

/*Function Des:获取当前设备内存数量
  Input:N/A
  Output:N/A
  Return:-1:系统错误， 其他则为内存数量
*/
int uoshwinfo_v2_raw_get_memery_num();

/*Function Des:获取当前设备内存信息
  Input:N/A
  Output:pmeminfo--内存信息数据结构；如果获取失败则数据内容为空
  Return:N/A
*/
void uoshwinfo_v2_raw_get_memery(uoshwinfo_v2_raw_mem *pmeminfo);

/*Function Des:获取当前设备网卡数量
  Input:N/A
  Output:N/A
  Return:-1:系统错误， 其他则为网卡数量
*/
int uoshwinfo_v2_raw_get_net_num();

/*Function Des:获取当前设备网卡信息
  Input:N/A
  Output:pnetinfo--网卡信息数据结构；如果获取失败则数据内容为空
  Return:N/A
*/
void uoshwinfo_v2_raw_get_net(uoshwinfo_v2_raw_net *pnetinfo);

/*Function Des:获取当前设备虚拟机信息
  Input:N/A
  Output:pvminfo--虚拟机信息数据结构；
                  成员变量type说明：
				  error: 获取虚拟机信息失败
				  none:  不是虚拟机
				  qemu,kvm....: 具体虚拟机类型
  Return:N/A
*/
int uoshwinfo_v2_raw_get_vm(uoshwinfo_v2_raw_vm *pvminfo);

/*Function Des:获取当前OEM的serial
  Input:N/A
  Output:N/A
  Return:-1:不存在该文件，也就是说不是oem机器，   其他：正常
*/
int uoshwinfo_v2_raw_get_msdm(uoshwinfo_v2_raw_msdm *pMsdmInfo);


int uoshwinfo_v2_raw_get_version(uoshwinfo_v2_raw_version *pversioninfo);

/**
 * @brief Get the root dir object
 * 
 * @param nameInfo 
 * @return int 
 */
int uoshwinfo_get_rootdir(uoshwinfo_v2_raw_rootdisk *nameInfo);


/**
 * @name    从cmdline获取根目录所在硬盘
 * @return  success: true    error: false
*/
int uoshwinfo_get_rootdir_for_cmdline(uoshwinfo_v2_raw_rootdisk *nameInfo);

#ifdef __cplusplus
}
#endif
#endif //#ifndef _UOSDEVICEH_V2_RAW_H_
